@@ -10,17 +10,18 @@ OC.L10N.register(
     "New" : "Nove",
     "New folder" : "Nove dossier",
     "Search" : "Cercar",
+    "_%n folder_::_%n folders_" : ["%n dossier","%n dossiers"],
     "Rename folder" : "Renominar dossier",
     "Cancel" : "Cancellar",
     "Recent" : "Recente",
     "Shared with you" : "Compartite con te",
     "Files" : "Files",
     "Delete" : "Deler",
-    "Settings" : "Configurationes",
     "Title" : "Titulo",
     "Link" : "Ligamine",
     "Tags" : "Etiquettas",
     "Notes" : "Notas",
+    "Open file" : "Aperir file",
     "Owner" : "Owner",
     "Sharing" : "Compartente",
     "Share link" : "Compartir ligamine",
