@@ -9,6 +9,8 @@
 namespace OCA\Bookmarks\Db;
 
 use OCA\Bookmarks\Events\BeforeDeleteEvent;
+use OCA\Bookmarks\Events\InsertEvent;
+use OCA\Bookmarks\Events\ManipulateEvent;
 use OCA\Bookmarks\Exception\AlreadyExistsError;
 use OCA\Bookmarks\Exception\UrlParseError;
 use OCA\Bookmarks\Exception\UserLimitExceededError;
@@ -74,6 +76,9 @@ class BookmarkMapper extends QBMapper {
 	 */
 	private $shareMapper;
 
+	/** @var array<string,int> */
+	private array $userBookmarkCount = [];
+
 	/**
 	 * BookmarkMapper constructor.
 	 *
@@ -107,6 +112,7 @@ class BookmarkMapper extends QBMapper {
 			->select('*')
 			->from('bookmarks')
 			->where($qb->expr()->eq('user_id', $qb->createParameter('user_id')))
+			->andWhere($qb->expr()->eq('url_hash', $qb->createParameter('url_hash')))
 			->andWhere($qb->expr()->eq('url', $qb->createParameter('url')));
 		return $qb;
 	}
@@ -130,7 +136,8 @@ class BookmarkMapper extends QBMapper {
 		$qb = $this->findByUrlQuery;
 		$qb->setParameters([
 			'user_id' => $userId,
-			'url' => $url
+			'url' => $url,
+			'url_hash' => hash('xxh128', $url),
 		]);
 		return $this->findEntity($qb);
 	}
@@ -139,17 +146,22 @@ class BookmarkMapper extends QBMapper {
 	 * @param string $userId
 	 * @throws DoesNotExistException
 	 * @throws MultipleObjectsReturnedException
+	 * @throws Exception
 	 */
 	public function deleteAll(string $userId): void {
 		$qb = $this->db->getQueryBuilder();
 		$qb->select('b.id')
 			->from('bookmarks', 'b')
 			->where($qb->expr()->eq('b.user_id', $qb->createPositionalParameter($userId)));
-		$orphanedBookmarks = $qb->execute();
-		while ($bookmark = $orphanedBookmarks->fetchColumn()) {
+		$result = $qb->executeQuery();
+		while ($bookmark = $result->fetchOne()) {
 			$bm = $this->find($bookmark);
 			$this->delete($bm);
 		}
+		if (isset($this->userBookmarkCount[$userId])) {
+			$this->userBookmarkCount[$userId] = 0;
+		}
+		$result->closeCursor();
 	}
 
 	/**
@@ -185,7 +197,7 @@ class BookmarkMapper extends QBMapper {
 		$qb
 			->select(Bookmark::$columns)
 			->from('bookmarks')
-			->where($qb->expr()->eq('id', $qb->createNamedParameter($id)));
+			->where($qb->expr()->eq('id', $qb->createNamedParameter($id, IQueryBuilder::PARAM_INT)));
 
 		return $this->findEntity($qb);
 	}
@@ -257,6 +269,19 @@ class BookmarkMapper extends QBMapper {
 		return $entities;
 	}
 
+	/**
+	 * @throws \OCP\DB\Exception
+	 */
+	protected function getIteratorWithRawQuery(string $query, array $params, array $types): \Generator {
+		$cursor = $this->db->executeQuery($query, $params, $types);
+
+		while ($row = $cursor->fetch()) {
+			yield $this->mapRowToEntity($row);
+		}
+
+		$cursor->closeCursor();
+	}
+
 	/**
 	 * Common table expression that lists all items in a given folder, recursively
 	 * @param int $folderId
@@ -384,6 +409,7 @@ class BookmarkMapper extends QBMapper {
 	private function _filterUrl(IQueryBuilder $qb, QueryParameters $params): void {
 		if (($url = $params->getUrl()) !== null) {
 			$normalized = $this->urlNormalizer->normalize($url);
+			$qb->andWhere($qb->expr()->eq('b.url_hash', $qb->createPositionalParameter(hash('xxh128', $normalized))));
 			$qb->andWhere($qb->expr()->eq('b.url', $qb->createPositionalParameter($normalized)));
 		}
 	}
@@ -454,9 +480,10 @@ class BookmarkMapper extends QBMapper {
 	private function _filterDuplicated(IQueryBuilder $qb, QueryParameters $params) {
 		if ($params->getDuplicated()) {
 			$subQuery = $this->db->getQueryBuilder();
+			$subQuery->automaticTablePrefix(false);
 			$subQuery->select('trdup.parent_folder')
-				->from('*PREFIX*bookmarks_tree', 'trdup')
-				->where($subQuery->expr()->eq('b.id', 'trdup.id'))
+				->from('folder_tree', 'trdup')
+				->where($subQuery->expr()->eq('b.id', 'trdup.item_id'))
 				->andWhere($subQuery->expr()->neq('trdup.parent_folder', 'tree.parent_folder'))
 				->andWhere($subQuery->expr()->eq('trdup.type', $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK)));
 			$qb->andWhere($qb->createFunction('EXISTS(' . $subQuery->getSQL() . ')'));
@@ -510,6 +537,38 @@ class BookmarkMapper extends QBMapper {
 		}
 	}
 
+	/**
+	 * @param string $userId
+	 * @return int
+	 * @throws Exception
+	 */
+	public function countDeleted(string $userId): int {
+		$qb = $this->db->getQueryBuilder();
+		$qb->selectAlias($qb->func()->count('b.id'), 'count');
+		$qb
+			->from('bookmarks', 'b')
+			->innerJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK))
+			->where($qb->expr()->eq('b.user_id', $qb->createPositionalParameter($userId)))
+			->andWhere($qb->expr()->isNotNull('tr.soft_deleted_at'));
+		$result = $qb->executeQuery();
+		$userOwnerDeletedCount = $result->fetch(PDO::FETCH_COLUMN);
+		$result->closeCursor();
+
+		$qb = $this->db->getQueryBuilder();
+		$qb->selectAlias($qb->func()->count('b.id'), 'count');
+		$qb
+			->from('bookmarks', 'b')
+			->innerJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK))
+			->innerJoin('tr', 'bookmarks_shared_folders', 'sf', $qb->expr()->eq('tr.parent_folder', 'sf.folder_id'))
+			->where($qb->expr()->eq('sf.user_id', $qb->createPositionalParameter($userId)))
+			->andWhere($qb->expr()->isNotNull('tr.soft_deleted_at'));
+		$result = $qb->executeQuery();
+		$foreignDeletedCount = $result->fetch(PDO::FETCH_COLUMN);
+		$result->closeCursor();
+
+		return $userOwnerDeletedCount + $foreignDeletedCount;
+	}
+
 	/**
 	 * @param string $userId
 	 * @return int
@@ -517,23 +576,28 @@ class BookmarkMapper extends QBMapper {
 	public function countArchived(string $userId): int {
 		$qb = $this->db->getQueryBuilder();
 		$qb->selectAlias($qb->func()->count('b.id'), 'count');
-
 		$qb
 			->from('bookmarks', 'b')
-			->leftJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
-			->leftJoin('tr', 'bookmarks_shared_folders', 'sf', $qb->expr()->eq('tr.parent_folder', 'sf.folder_id'))
-			->where(
-				$qb->expr()->andX(
-					$qb->expr()->orX(
-						$qb->expr()->eq('b.user_id', $qb->createPositionalParameter($userId)),
-						$qb->expr()->eq('sf.user_id', $qb->createPositionalParameter($userId))
-					),
-					$qb->expr()->in('b.user_id', array_map([$qb, 'createPositionalParameter'], array_merge($this->_findSharersFor($userId), [$userId])))
-				)
-			)
+			->innerJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
+			->where($qb->expr()->eq('b.user_id', $qb->createPositionalParameter($userId)))
 			->andWhere($qb->expr()->isNotNull('b.archived_file'));
+		$result = $qb->executeQuery();
+		$userOwnerArchivedCount = $result->fetch(PDO::FETCH_COLUMN);
+		$result->closeCursor();
 
-		return $qb->execute()->fetch(PDO::FETCH_COLUMN);
+		$qb = $this->db->getQueryBuilder();
+		$qb->selectAlias($qb->func()->count('b.id'), 'count');
+		$qb
+			->from('bookmarks', 'b')
+			->innerJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
+			->innerJoin('tr', 'bookmarks_shared_folders', 'sf', $qb->expr()->eq('tr.parent_folder', 'sf.folder_id'))
+			->where($qb->expr()->eq('sf.user_id', $qb->createPositionalParameter($userId)))
+			->andWhere($qb->expr()->isNotNull('b.archived_file'));
+		$result = $qb->executeQuery();
+		$foreignArchivedCount = $result->fetch(PDO::FETCH_COLUMN);
+		$result->closeCursor();
+
+		return $userOwnerArchivedCount + $foreignArchivedCount;
 	}
 
 	/**
@@ -544,22 +608,28 @@ class BookmarkMapper extends QBMapper {
 	public function countAllClicks(string $userId): int {
 		$qb = $this->db->getQueryBuilder();
 		$qb->selectAlias($qb->func()->sum('b.clickcount'), 'count');
-
 		$qb
 			->from('bookmarks', 'b')
-			->leftJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
-			->leftJoin('tr', 'bookmarks_shared_folders', 'sf', $qb->expr()->eq('tr.parent_folder', 'sf.folder_id'))
-			->where(
-				$qb->expr()->andX(
-					$qb->expr()->orX(
-						$qb->expr()->eq('b.user_id', $qb->createPositionalParameter($userId)),
-						$qb->expr()->eq('sf.user_id', $qb->createPositionalParameter($userId))
-					),
-					$qb->expr()->in('b.user_id', array_map([$qb, 'createPositionalParameter'], array_merge($this->_findSharersFor($userId), [$userId])))
-				)
-			);
+			->innerJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
+			->where($qb->expr()->eq('b.user_id', $qb->createPositionalParameter($userId)))
+			->andWhere($qb->expr()->neq('b.clickcount', $qb->createPositionalParameter(0, IQueryBuilder::PARAM_INT)));
+		$result = $qb->executeQuery();
+		$userOwnerClickCount = $result->fetch(PDO::FETCH_COLUMN);
+		$result->closeCursor();
 
-		return $qb->execute()->fetch(PDO::FETCH_COLUMN) ?? 0;
+		$qb = $this->db->getQueryBuilder();
+		$qb->selectAlias($qb->func()->sum('b.clickcount'), 'count');
+		$qb
+			->from('bookmarks', 'b')
+			->innerJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
+			->innerJoin('tr', 'bookmarks_shared_folders', 'sf', $qb->expr()->eq('tr.parent_folder', 'sf.folder_id'))
+			->where($qb->expr()->eq('sf.user_id', $qb->createPositionalParameter($userId)))
+			->andWhere($qb->expr()->neq('b.clickcount', $qb->createPositionalParameter(0, IQueryBuilder::PARAM_INT)));
+		$result = $qb->executeQuery();
+		$foreignClickCount = $result->fetch(PDO::FETCH_COLUMN);
+		$result->closeCursor();
+
+		return $userOwnerClickCount + $foreignClickCount;
 	}
 
 	/**
@@ -570,23 +640,28 @@ class BookmarkMapper extends QBMapper {
 	public function countWithClicks(string $userId): int {
 		$qb = $this->db->getQueryBuilder();
 		$qb->selectAlias($qb->func()->count('b.id'), 'count');
-
 		$qb
 			->from('bookmarks', 'b')
-			->leftJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
-			->leftJoin('tr', 'bookmarks_shared_folders', 'sf', $qb->expr()->eq('tr.parent_folder', 'sf.folder_id'))
-			->where(
-				$qb->expr()->andX(
-					$qb->expr()->orX(
-						$qb->expr()->eq('b.user_id', $qb->createPositionalParameter($userId)),
-						$qb->expr()->eq('sf.user_id', $qb->createPositionalParameter($userId))
-					),
-					$qb->expr()->in('b.user_id', array_map([$qb, 'createPositionalParameter'], array_merge($this->_findSharersFor($userId), [$userId])))
-				)
-			)
+			->innerJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
+			->where($qb->expr()->eq('b.user_id', $qb->createPositionalParameter($userId)))
 			->andWhere($qb->expr()->neq('b.clickcount', $qb->createPositionalParameter(0, IQueryBuilder::PARAM_INT)));
+		$result = $qb->executeQuery();
+		$userOwnerWithClicksCount = $result->fetch(PDO::FETCH_COLUMN);
+		$result->closeCursor();
 
-		return $qb->execute()->fetch(PDO::FETCH_COLUMN);
+		$qb = $this->db->getQueryBuilder();
+		$qb->selectAlias($qb->func()->count('b.id'), 'count');
+		$qb
+			->from('bookmarks', 'b')
+			->innerJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
+			->innerJoin('tr', 'bookmarks_shared_folders', 'sf', $qb->expr()->eq('tr.parent_folder', 'sf.folder_id'))
+			->where($qb->expr()->eq('sf.user_id', $qb->createPositionalParameter($userId)))
+			->andWhere($qb->expr()->neq('b.clickcount', $qb->createPositionalParameter(0, IQueryBuilder::PARAM_INT)));
+		$result = $qb->executeQuery();
+		$foreignWithClicksCount = $result->fetch(PDO::FETCH_COLUMN);
+		$result->closeCursor();
+
+		return $userOwnerWithClicksCount + $foreignWithClicksCount;
 	}
 
 	/**
@@ -597,55 +672,74 @@ class BookmarkMapper extends QBMapper {
 	public function countUnavailable(string $userId): int {
 		$qb = $this->db->getQueryBuilder();
 		$qb->selectAlias($qb->func()->count('b.id'), 'count');
-
 		$qb
 			->from('bookmarks', 'b')
-			->leftJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
-			->leftJoin('tr', 'bookmarks_shared_folders', 'sf', $qb->expr()->eq('tr.parent_folder', 'sf.folder_id'))
-			->where(
-				$qb->expr()->andX(
-					$qb->expr()->orX(
-						$qb->expr()->eq('b.user_id', $qb->createPositionalParameter($userId)),
-						$qb->expr()->eq('sf.user_id', $qb->createPositionalParameter($userId))
-					),
-					$qb->expr()->in('b.user_id', array_map([$qb, 'createPositionalParameter'], array_merge($this->_findSharersFor($userId), [$userId])))
-				)
-			)
+			->innerJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
+			->where($qb->expr()->eq('b.user_id', $qb->createPositionalParameter($userId)))
 			->andWhere($qb->expr()->eq('b.available', $qb->createPositionalParameter(false, IQueryBuilder::PARAM_BOOL)));
+		$result = $qb->executeQuery();
+		$userOwnerUnavailableCount = $result->fetch(PDO::FETCH_COLUMN);
+		$result->closeCursor();
 
-		return $qb->execute()->fetch(PDO::FETCH_COLUMN);
+		$qb = $this->db->getQueryBuilder();
+		$qb->selectAlias($qb->func()->count('b.id'), 'count');
+		$qb
+			->from('bookmarks', 'b')
+			->innerJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
+			->innerJoin('tr', 'bookmarks_shared_folders', 'sf', $qb->expr()->eq('tr.parent_folder', 'sf.folder_id'))
+			->where($qb->expr()->eq('sf.user_id', $qb->createPositionalParameter($userId)))
+			->andWhere($qb->expr()->eq('b.available', $qb->createPositionalParameter(false, IQueryBuilder::PARAM_BOOL)));
+		$result = $qb->executeQuery();
+		$foreignUnavailableCount = $result->fetch(PDO::FETCH_COLUMN);
+		$result->closeCursor();
+
+		return $userOwnerUnavailableCount + $foreignUnavailableCount;
 	}
 
 	/**
 	 * @param string $userId
 	 * @return int
+	 * @throws Exception
 	 */
 	public function countDuplicated(string $userId): int {
 		$qb = $this->db->getQueryBuilder();
 		$qb->selectDistinct($qb->func()->count('b.id'));
-
 		$qb
 			->from('bookmarks', 'b')
-			->leftJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
-			->leftJoin('tr', 'bookmarks_shared_folders', 'sf', $qb->expr()->eq('tr.parent_folder', 'sf.folder_id'))
-			->where(
-				$qb->expr()->andX(
-					$qb->expr()->orX(
-						$qb->expr()->eq('b.user_id', $qb->createPositionalParameter($userId)),
-						$qb->expr()->eq('sf.user_id', $qb->createPositionalParameter($userId))
-					),
-					$qb->expr()->in('b.user_id', array_map([$qb, 'createPositionalParameter'], array_merge($this->_findSharersFor($userId), [$userId]))),
-				)
-			);
+			->innerJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
+			->where($qb->expr()->eq('b.user_id', $qb->createPositionalParameter($userId)));
 		$subQuery = $this->db->getQueryBuilder();
 		$subQuery->select('trdup.parent_folder')
 			->from('bookmarks_tree', 'trdup')
 			->where($subQuery->expr()->eq('b.id', 'trdup.id'))
 			->andWhere($subQuery->expr()->neq('trdup.parent_folder', 'tr.parent_folder'))
-			->andWhere($subQuery->expr()->eq('trdup.type', $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK)));
+			->andWhere($subQuery->expr()->eq('trdup.type', $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK)))
+			->andWhere($subQuery->expr()->isNull('trdup.soft_deleted_at'));
 		$qb->andWhere($qb->createFunction('EXISTS(' . $subQuery->getSQL() . ')'));
+		$result = $qb->executeQuery();
+		$userOwnerDuplicatesCount = $result->fetch(PDO::FETCH_COLUMN);
+		$result->closeCursor();
 
-		return $qb->execute()->fetch(PDO::FETCH_COLUMN);
+		$qb = $this->db->getQueryBuilder();
+		$qb->selectDistinct($qb->func()->count('b.id'));
+		$qb
+			->from('bookmarks', 'b')
+			->innerJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
+			->innerJoin('tr', 'bookmarks_shared_folders', 'sf', $qb->expr()->eq('tr.parent_folder', 'sf.folder_id'))
+			->where($qb->expr()->eq('sf.user_id', $qb->createPositionalParameter($userId)));
+		$subQuery = $this->db->getQueryBuilder();
+		$subQuery->select('trdup.parent_folder')
+			->from('bookmarks_tree', 'trdup')
+			->where($subQuery->expr()->eq('b.id', 'trdup.id'))
+			->andWhere($subQuery->expr()->neq('trdup.parent_folder', 'tr.parent_folder'))
+			->andWhere($subQuery->expr()->eq('trdup.type', $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK)))
+			->andWhere($subQuery->expr()->isNull('trdup.soft_deleted_at'));
+		$qb->andWhere($qb->createFunction('EXISTS(' . $subQuery->getSQL() . ')'));
+		$result = $qb->executeQuery();
+		$foreignDuplicatesCount = $result->fetch(PDO::FETCH_COLUMN);
+		$result->closeCursor();
+
+		return $userOwnerDuplicatesCount + $foreignDuplicatesCount;
 	}
 
 	/**
@@ -686,7 +780,7 @@ class BookmarkMapper extends QBMapper {
 
 		$qb
 			->from('*PREFIX*bookmarks', 'b')
-			->join('b', 'folder_tree', 'tree', 'tree.item_id = b.id AND tree.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tree.soft_deleted_at is NULL');
+			->innerJoin('b', 'folder_tree', 'tree', 'tree.item_id = b.id AND tree.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tree.soft_deleted_at is NULL');
 
 
 		$this->_filterUrl($qb, $queryParams);
@@ -718,7 +812,7 @@ class BookmarkMapper extends QBMapper {
 		$qb = $this->db->getQueryBuilder();
 		$qb->select(Bookmark::$columns);
 		$qb->from('bookmarks', 'b');
-		$qb->where($qb->expr()->lt('last_preview', $qb->createPositionalParameter($this->time->getTime() - $stalePeriod)));
+		$qb->where($qb->expr()->lt('last_preview', $qb->createPositionalParameter($this->time->getTime() - $stalePeriod, IQueryBuilder::PARAM_INT)));
 		$qb->orWhere($qb->expr()->isNull('last_preview'));
 		$qb->setMaxResults($limit);
 		return $this->findEntities($qb);
@@ -731,7 +825,7 @@ class BookmarkMapper extends QBMapper {
 	public function clearLastPreviews(): void {
 		$qb = $this->db->getQueryBuilder();
 		$qb->update('bookmarks');
-		$qb->set('last_preview', $qb->createNamedParameter(0));
+		$qb->set('last_preview', $qb->createNamedParameter(0, IQueryBuilder::PARAM_INT));
 		$qb->executeStatement();
 	}
 
@@ -750,12 +844,15 @@ class BookmarkMapper extends QBMapper {
 		);
 
 		$returnedEntity = parent::delete($entity);
+		if (isset($this->userBookmarkCount[$entity->getUserId()])) {
+			$this->userBookmarkCount[$entity->getUserId()] -= 1;
+		}
 
 		$id = $entity->getId();
 
 		$qb = $this->deleteTagsQuery;
 		$qb->setParameter('id', $id);
-		$qb->execute();
+		$qb->executeStatement();
 
 		return $returnedEntity;
 	}
@@ -771,8 +868,11 @@ class BookmarkMapper extends QBMapper {
 		if ($entity->isWebLink()) {
 			$entity->setUrl($this->urlNormalizer->normalize($entity->getUrl()));
 		}
+		$entity->setUrlHash(hash('xxh128', $entity->getUrl()));
 		$entity->setLastmodified(time());
-		return parent::update($entity);
+		parent::update($entity);
+		$this->eventDispatcher->dispatchTyped(new ManipulateEvent('bookmark', $entity->getId()));
+		return $entity;
 	}
 
 	/**
@@ -793,24 +893,38 @@ class BookmarkMapper extends QBMapper {
 		if ($entity->isWebLink()) {
 			$entity->setUrl($this->urlNormalizer->normalize($entity->getUrl()));
 		}
-
-		if ($entity->getAdded() === null) {
-			$entity->setAdded(time());
+		$entityToInsert = new Bookmark();
+		foreach ($entity->getUpdatedFields() as $field => $_value) {
+			$value = $entity->{'get' . ucfirst($field)}();
+			$entityToInsert->{'set' . ucfirst($field)}($value);
 		}
-		$entity->setLastmodified(time());
-		$entity->setLastPreview(0);
-		$entity->setClickcount(0);
+
+		$entityToInsert->setUrlHash(hash('xxh128', $entity->getUrl()));
+
+		if ($entityToInsert->getAdded() === null) {
+			$entityToInsert->setAdded(time());
+		}
+		$entityToInsert->setLastmodified(time());
+		$entityToInsert->setLastPreview(0);
+		$entityToInsert->setClickcount(0);
 
 		try {
-			$this->findByUrl($entity->getUserId(), $entity->getUrl());
-		} catch (DoesNotExistException $e) {
-			parent::insert($entity);
-			return $entity;
-		} catch (MultipleObjectsReturnedException $e) {
-			// noop
+			parent::insert($entityToInsert);
+			if (isset($this->userBookmarkCount[$entityToInsert->getUserId()])) {
+				$this->userBookmarkCount[$entityToInsert->getUserId()] += 1;
+			}
+			$this->eventDispatcher->dispatchTyped(new InsertEvent('bookmark', $entityToInsert->getId()));
+			return $entityToInsert;
+		} catch (Exception $e) {
+			if ($e->getReason() === Exception::REASON_UNIQUE_CONSTRAINT_VIOLATION) {
+				if ($this->db->inTransaction()) {
+					$this->db->commit();
+					$this->db->beginTransaction();
+				}
+				throw new AlreadyExistsError('A bookmark with this URL already exists');
+			}
+			throw $e;
 		}
-
-		throw new AlreadyExistsError('A bookmark with this URL already exists');
 	}
 
 	/**
@@ -824,28 +938,42 @@ class BookmarkMapper extends QBMapper {
 	public function insertOrUpdate(Entity $entity): Bookmark {
 		try {
 			$newEntity = $this->insert($entity);
-		} catch (AlreadyExistsError $e) {
-			$bookmark = $this->findByUrl($entity->getUserId(), $entity->getUrl());
-			$entity->setId($bookmark->getId());
-			$newEntity = $this->update($entity);
+		} catch (AlreadyExistsError) {
+			try {
+				$bookmark = $this->findByUrl($entity->getUserId(), $entity->getUrl());
+				foreach ($entity->getUpdatedFields() as $field => $_value) {
+					$value = $entity->{'get' . ucfirst($field)}();
+					$bookmark->{'set' . ucfirst($field)}($value);
+				}
+				$newEntity = $this->update($bookmark);
+			} catch (DoesNotExistException) {
+				$newEntity = $this->insert($entity);
+			}
 		}
 
 		return $newEntity;
 	}
 
 	/**
-	 * @param $userId
-	 * @param string $userId
-	 *
-	 * @return int
+	 * @throws Exception
 	 */
 	public function countBookmarksOfUser(string $userId) : int {
+		if (isset($this->userBookmarkCount[$userId])) {
+			return $this->userBookmarkCount[$userId];
+		}
 		$qb = $this->db->getQueryBuilder();
 		$qb
 			->select($qb->func()->count('id'))
 			->from('bookmarks')
 			->where($qb->expr()->eq('user_id', $qb->createPositionalParameter($userId)));
-		return $qb->execute()->fetch(PDO::FETCH_COLUMN);
+		$result = $qb->executeQuery();
+		$count = $result->fetchOne();
+		if ($count === false) {
+			$count = 0;
+		}
+		$result->closeCursor();
+		$this->userBookmarkCount[$userId] = (int)$count;
+		return (int)$count;
 	}
 
 	/**
@@ -903,4 +1031,41 @@ class BookmarkMapper extends QBMapper {
 			return  $share->getOwner();
 		}, $this->shareMapper->findByUser($userId));
 	}
+
+	public function getIterator(string $userId, QueryParameters $queryParams): \Generator {
+		$rootFolder = $this->folderMapper->findRootFolder($userId);
+		// gives us all bookmarks in this folder, recursively
+		[$cte, $params, $paramTypes] = $this->_generateCTE($rootFolder->getId(), $queryParams->getSoftDeletedFolders());
+
+		$qb = $this->db->getQueryBuilder();
+		$bookmark_cols = array_map(static function ($c) {
+			return 'b.' . $c;
+		}, Bookmark::$columns);
+
+		$qb->select($bookmark_cols);
+		$qb->groupBy($bookmark_cols);
+
+		$qb->automaticTablePrefix(false);
+
+		$qb
+			->from('*PREFIX*bookmarks', 'b')
+			->innerJoin('b', 'folder_tree', 'tree', 'tree.item_id = b.id AND tree.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK)
+				. ($queryParams->getSoftDeleted() ? ' AND tree.soft_deleted_at is NOT NULL' : ' AND tree.soft_deleted_at is NULL'));
+
+		$this->_filterUrl($qb, $queryParams);
+		$this->_filterArchived($qb, $queryParams);
+		$this->_filterUnavailable($qb, $queryParams);
+		$this->_filterDuplicated($qb, $queryParams);
+		$this->_filterFolder($qb, $queryParams);
+		$this->_filterTags($qb, $queryParams);
+		$this->_filterUntagged($qb, $queryParams);
+		$this->_filterSearch($qb, $queryParams);
+		$this->_sortAndPaginate($qb, $queryParams);
+
+		$finalQuery = $cte . ' ' . $qb->getSQL();
+
+		$params = array_merge($params, $qb->getParameters());
+		$paramTypes = array_merge($paramTypes, $qb->getParameterTypes());
+		return $this->getIteratorWithRawQuery($finalQuery, $params, $paramTypes);
+	}
 }
